=begin header
    Gimp-Ruby - allows the writing of Gimp plugins in Ruby.
    Copyright (C) 2001  Masahiro Sakai
 
          Masahiro Sakai                  <zvm01052@nifty.ne.jp>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
=end

require "gimp"
require 'gimpui'

module Gimp

  RF_FLOAT       = :float
  RF_STRING      = :string
  RF_COLOR       = :color
  RF_REGION      = :region
  RF_IMAGE       = :image
  RF_LAYER       = :layer
  RF_CHANNEL     = :channel
  RF_DRAWABLE    = :drawable
  RF_TOGGLE      = :toggle
  RF_ADJUSTMENT  = :adjustment
  RF_OPTION      = :option
  RF_FONT        = :font
  RF_FILENAME    = :file
  RF_BRUSH       = :brush
  RF_PATTERN     = :pattern
  RF_GRADIENT    = :gradient

  #RF_DISPLAY     = :display
  #RF_SELECTION   = :selection
  #RF_BOUNDARY    = :boundary
  #RF_PATH        = :path
  #RF_STATUS      = :status

  RF_OBJECT      = :object
  RF_INT8        = :int8
  RF_INT16       = :int16
  RF_INT32       = :int32
  RF_INT         = RF_INT32
  RF_INT8ARRAY   = :int8array
  RF_INT16ARRAY  = :int16array
  RF_INT32ARRAY  = :int32array
  RF_INTARRAY    = RF_INT32ARRAY
  RF_FLOATARRAY  = :floatarray
  RF_STRINGARRAY = :stringarray


  TYPE_MAPPING = {
    RF_FLOAT       => PDB_FLOAT,
    RF_STRING      => PDB_STRING,
    RF_COLOR       => PDB_COLOR,
    RF_REGION      => PDB_REGION,
    RF_IMAGE       => PDB_IMAGE,
    RF_LAYER       => PDB_LAYER,
    RF_CHANNEL     => PDB_CHANNEL,
    RF_DRAWABLE    => PDB_DRAWABLE,
    RF_TOGGLE      => PDB_INT32,
    RF_ADJUSTMENT  => PDB_INT32,
    RF_OPTION      => PDB_INT32,
    RF_FONT        => PDB_STRING,
    RF_FILENAME    => PDB_STRING,
    RF_BRUSH       => PDB_STRING,
    RF_PATTERN     => PDB_STRING,
    RF_GRADIENT    => PDB_STRING,
    RF_OBJECT      => PDB_STRING,

    RF_INT8        => PDB_INT8,
    RF_INT16       => PDB_INT16,
    RF_INT32       => PDB_INT32,
    RF_INT8ARRAY   => PDB_INT8ARRAY,
    RF_INT16ARRAY  => PDB_INT16ARRAY,
    RF_INT32ARRAY  => PDB_INT32ARRAY,
    RF_FLOATARRAY  => PDB_FLOATARRAY,
    RF_STRINGARRAY => PDB_STRINGARRAY,
  }

###############################################################################

  # Script-Fu$B$@$H$3$&$$$&=gHV(B
  # name, menupath, help. copyright, date, imagetypes, params, results

  def register(name, blurb, help, author, copyright, date, menupath,
               imagetypes, params = [], results = [], &proc)

    if (/^<Image>\// =~ menupath) and (params.size >= 2) and
        (params[0][0] == RF_IMAGE) and (params[1][0] == RF_DRAWABLE)
      plugin_type = PLUGIN
    else
      plugin_type = EXTENSION
    end

    Fu::RegisteredPlugins[name] = [blurb, help, author, copyright, date,
                                   menupath, imagetypes, plugin_type,
                                   params, results, proc]
  end

###############################################################################

  module Fu
    RegisteredPlugins = {}

    def query
      RegisteredPlugins.each{|key, value|
        blurb, help, author, copyright, date,
          menupath, imagetypes, plugin_type,
          params, results, proc = *value

        params = params.collect{ |item|
          [TYPE_MAPPING[item[0]], item[1], item[2]]
        }
        results = results.collect{ |item|
          [TYPE_MAPPING[item[0]], item[1], item[2]]
        }

        params = [[PDB_INT32, "run_mode", "Interactive, Non-Interactive"]] + params

        install_procedure(key, blurb, help, author, copyright,
                          date, menupath, imagetypes, plugin_type,
                          params, results)
      }
    end
    module_function :query


    def run(proc_name, run_mode, *params)
      plugin_type = RegisteredPlugins[proc_name][7]
      paramdefs   = RegisteredPlugins[proc_name][8]
      proc        = RegisteredPlugins[proc_name][10]

      paramdefs.each_with_index { |item, index|
        case item[0]
        when RF_TOGGLE
          if params[index].is_a? Integer
            params[index] = (params[index] != 0)
          end
        when RF_OBJECT
          if params[index].is_a? String
            params[index] = Marshal.load(params[index])
          end
        end
      }

      defaults = get_defaults(proc_name)
    
      case run_mode
      when RUN_INTERACTIVE
        values = interact(proc_name)
        if plugin_type == PLUGIN
          values.each_with_index { |item, index|
            params[index + 2] = item
          }
        else
          params = values
        end
      when RUN_WITH_LAST_VALS
        defaults.each_with_index { |item, index|
          params[index] = item if item
        }
      end

      *vals = proc.call(*params)
      Gimp::displays_flush() if run_mode != RUN_NONINTERACTIVE

      case vals.size
      when 0
        nil
      when 1
        vals[0]
      else
        vals
      end      
    end
    module_function :run


    def get_defaults(proc_name)
      blurb, help, author, copyright, date,
        menupath, imagetypes, plugin_type,
        params, results, proc = RegisteredPlugins[proc_name]

      params.collect{ |param|
        param[3]
      }
    end
    module_function :get_defaults

    def interact(proc_name)
      blurb, help, author, copyright, date,
        menupath, imagetypes, plugin_type,
        params, results, proc = RegisteredPlugins[proc_name]

      # short circuit for no parameters ...
      return [] if params.size == 0
      return [] if plugin_type == PLUGIN and params.size == 2

      dialog = Dialog::new(proc_name)
      #dialog.signal_connect("delete_event") { true }
      begin
        dialog.show
        Gtk::main()
        return dialog.values
      ensure
        dialog.destroy
      end
    end
    module_function :interact

  end

###############################################################################

  class <<PLUG_IN_INFO
    def query
      begin
        Fu::query
      rescue StandardError, ScriptError => e
        message("Ruby-Fu:\n" + e.inspect + "\n" + e.backtrace.join("\n"))
        raise
      end
    end

    def run(proc_name, *params)
      begin
        Fu::run(proc_name, *params)
      rescue PDB::Cancel, PDB::PassThrough
        return
      rescue StandardError, ScriptError => e
        message("Ruby-Fu:\n" + e.inspect + "\n" + e.backtrace.join("\n"))
        raise
      end
    end
  end

###############################################################################

  module Fu
    class Dialog < Gtk::Dialog

      class Entry
        attr_reader   :widget
        attr_accessor :value
      end

      class StringEntry < Entry
        def initialize(value, multiline = false)
          @multiline = multiline
          if @multiline
            @widget = Gtk::Text::new()
            @widget.insert_text(value, 0)
            @widget.set_editable(true)
          else
            @widget = Gtk::Entry::new()
            @widget.set_text(String(value))
          end
        end

        def value
          if @multiline
            @widget.get_chars(0, @widget.get_length)
          else
            @widget.get_text
          end
        end

        def value=(rhs)
          if @multiline
            @widget.delete_text
            @widget.insert_text(rhs, 0);
          else
            @widget.set_text(String(rhs))
          end
        end
      end

      class FloatEntry < StringEntry
        def value
          Float(super())
        end
      end

      class ArrayEntry < StringEntry
        def value
          eval(super())
        end
      end
      
      class ColorEntry < Entry
        def initialize(value = palette_get_foreground)
          @widget = Gimp::UI::ColorButton::new("color", 100, 20, value.to_a)
        end

        def value
          color = @widget.color
          Gimp::Color::new(color[0], color[1], color[2])
        end

        def value=(rhs)
          @widget.color = rhs
        end
      end

      class ToggleEntry < Entry
        def initialize(value)
          @widget = Gtk::CheckButton::new("") # FIXME
          @widget.active = value
        end

        def value
          @widget.active
        end

        def value=(rhs)
          @widget.active = rhs
        end
      end

      class AdjustmentEntry < Entry
        def initialize(value = 0,
                       lower = 0, upper = 100,
                       step_increment = 5, page_increment = 10,
                       page_size = 10,
                       is_spin = false)
          @adj = Gtk::Adjustment::new(value, lower, upper,
                                      step_increment, page_increment,
                                      page_size)
          if is_spin and (is_spin != 0)
            @widget = Gtk::SpinButton::new(@adj, 1, 0)
          else
            # FIXME: UI::scale_entry_new$B$GCV$-49$($k(B
            @widget = Gtk::HScale::new(@adj)
          end
        end
        
        def value
          @adj.value
        end

        def value=(rhs)
          @adj.value = rhs
        end
      end

      # FIXME
      class IntEntry < AdjustmentEntry
        def initialize(value)
          super(value, -10000000, +10000000, 1, 10, 10)
        end

        def value
          Integer(super())
        end
      end


      class OptionEntry < Entry
        def initialize(value = 0, *enums)
          @value = value
          @hash  = {}
          @menu  = Gtk::Menu::new()
          enums.each_with_index{ |item, index|
            menuitem = Gtk::MenuItem::new(item).show()
            @menu.append(menuitem)
            @hash[menuitem] = index
          }
          @menu.set_active(value)
          @widget = Gtk::OptionMenu::new().set_menu(@menu)
        end

        def value
          @hash[@menu.get_active]
        end

        def value=(rhs)
          @value = rhs
          @menu.set_active(rhs)
        end
      end

      class FontEntry < StringEntry # FIXME
      end
      
      class FileEntry < Entry
        def initialize(val = "")
          @widget = UI::FileSelection::new("", val, false, true)
        end

        def value
          @widget.filename
        end

        def value=(rhs)
          @widget.filename = rhs
        end
      end
      
      class BrushEntry < Entry
        def initialize(val = "")
          @value = val
          @widget = UI::brush_select_widget("Ruby-Fu Brush Selection",
                                            val, 100, 0, NORMAL_MODE) {
            |name, opacity, spacing, paint_mode, width, height, mask_data, closing|
            @value = name
          }
        end

        def value=(rhs)
          @value = rhs
          @widget.set_popup(rhs, 100, 0, NORMAL_MODE)
        end
      end
      
      class PatternEntry < Entry
        def initialize(val = "")
          @value = val
          @widget = UI::pattern_select_widget("Ruby-Fu Pattern Selection",
                                              val) {
            |name, width, height, bpp, mask_data, closing|
            @value = name
          }
        end

        def value=(rhs)
          @value = rhs
          @widget.set_popup(rhs, rhs)
        end
      end
      
      class GradientEntry < Entry
        def initialize(val = "")
          @value = val
          @widget = UI::gradient_select_widget("Ruby-Fu Gradient Selection",
                                               val) {
            |name, width, grad_data, dialog_closing|
            @value = name
          }
        end

        def value=(rhs)
          @value = rhs
          # ???
          @widget.set_popup(rhs, rhs)
        end
      end

      class ImageEntry < Entry
        def initialize(val = nil)
          val = Image::new_from_id(0) # FIXME
          @value = val
          @widget = UI::image_menu_new(nil, val) {
            |image|
            @value = image
          }
        end
      end

      class LayerEntry < Entry
        def initialize(val = nil)
          @value = val
          @widget = UI::layer_menu_new(nil, val) {
            |layer|
            @value = layer
          }
        end
      end      

      class ChannelEntry < Entry
        def initialize(val = nil)
          @value = val
          @widget = UI::channel_menu_new(nil, val) {
            |channel|
            @value = channel
          }
        end
      end

      class DrawableEntry < Entry
        def initialize(val = nil)
          @value = val
          @widget = UI::drawable_menu_new(nil, val) {
            |drawable|
            @value = drawable
          }
        end
      end

      EDIT_MAPPING = {
        RF_INT8        => IntEntry,
        RF_INT16       => IntEntry,
        RF_INT32       => IntEntry,
        RF_FLOAT       => FloatEntry,
        RF_STRING      => StringEntry,
        RF_INT8ARRAY   => ArrayEntry,
        RF_INT16ARRAY  => ArrayEntry,
        RF_INT32ARRAY  => ArrayEntry,
        RF_FLOATARRAY  => ArrayEntry,
        RF_STRINGARRAY => ArrayEntry,
        RF_COLOR       => ColorEntry,
        
        # FIXME
        # RF_REGION      => 
        RF_IMAGE       => ImageEntry,
        RF_LAYER       => LayerEntry,
        RF_CHANNEL     => ChannelEntry,
        RF_DRAWABLE    => DrawableEntry,
        
        RF_TOGGLE      => ToggleEntry,
        RF_ADJUSTMENT  => AdjustmentEntry,
        RF_OPTION      => OptionEntry,

        RF_FONT        => FontEntry,
        RF_FILENAME    => FileEntry,
        RF_BRUSH       => BrushEntry,
        RF_PATTERN     => PatternEntry,
        RF_GRADIENT    => GradientEntry,
      }

      attr :ok_flag

      def initialize(proc_name)
        blurb, help, author, copyright, date,
          menupath, imagetypes, plugin_type,
          params, results, proc = RegisteredPlugins[proc_name]

        if plugin_type == PLUGIN
          params = params[2...params.size]
        end
        
        super()
        set_title(proc_name)

        @about   = nil
        @ok_flag = false
        
        tooltips = Gtk::Tooltips::new()

        @notebook = Gtk::Notebook::new().show()
        self.vbox.pack_start(@notebook)

        ## Parameters

        frame = Gtk::Frame::new("").show()
        frame.border_width = 3
        @notebook.append_page(frame, Gtk::Label::new("Parameters").show())

        table = Gtk::Table::new(params.size, 2, FALSE).show()
        table.border_width = 3
        table.set_row_spacings(2)
        table.set_col_spacings(10)
        frame.add(table)

        defaults = Fu::get_defaults(proc_name)
        if plugin_type == PLUGIN
          defaults = defaults[2...defaults.size]
        end

        @entries = []
      
        params.each_with_index { |param, index|
          type = param[0]
          name = param[1]
          desc = param[2]
          def_val = defaults[index]
          
          label = Gtk::Label::new(desc).set_alignment(1.0, 0.5).show()
          table.attach(label, 0, 1, index, index + 1, Gtk::FILL)
  
          if param[4]
            entry = EDIT_MAPPING[type].new(def_val, *(param[4]))
          else
            entry = EDIT_MAPPING[type].new(def_val)
          end
          
          table.attach(entry.widget, 1, 2, index, index + 1) # FIXME
          tooltips.set_tip(entry.widget, name, nil)     
          entry.widget.show
          @entries.push(entry)
        }

        ## Buttons

        b = Gtk::Button::new("OK").show()
        b.set_flags(Gtk::Widget::CAN_DEFAULT)
        action_area.pack_start(b)
        b.grab_default()
        b.signal_connect("clicked") {
          @ok_flag = true
          delete_event()
        }

        b = Gtk::Button::new("Cancel").show()
        b.set_flags(Gtk::Widget::CAN_DEFAULT)
        action_area.pack_start(b)
        b.signal_connect("clicked") {
          delete_event()
          raise PDB::Cancel, proc_name
        }

        ## About

        vbox = Gtk::VBox::new(false, 15).show()
        @notebook.append_page(vbox, Gtk::Label::new("About").show())
        vbox.realize
        vbox.pack_start(get_logo(vbox.get_colormap).show(), true)

        table = Gtk::Table::new(2, 5).show()
        table.set_row_spacings(2)
        table.set_col_spacings(10)
        vbox.pack_start(table)

        [
          ["Name:", proc_name],
          ["Blurb:", blurb],
          ["Author:", author],
          ["Copyright:", copyright],
          ["Date:", date]
        ].each_with_index { |item, index|
          row = index
          label = Gtk::Label::new(item[0]).set_alignment(1.0, 0.5).show()
          table.attach(label, 0, 1, row, row + 1)
          label = Gtk::Label::new(item[1]).set_alignment(0.0, 0.5).show()
          label.set_line_wrap(true)
          table.attach(label, 1, 2, row, row + 1)
        }  

        textbox = Gtk::Text::new().show().set_editable(false)
        textbox.insert_text(help, 0)
        textbox.set_usize(100, 50)
        vbox.pack_start(textbox)
      end

      def delete_event()
        hide()
        Gtk::main_quit()
        false
      end

      def values
        @entries.collect{ |item| item.value }
      end

      LOGO_IMAGE = [
"50 35 17 1",
"       c None",
".      c #021423",
"+      c #FDE3EC",
"@      c #F596D7",
"O      c #770C45",
"$      c #794241",
"%      c #A08254",
"&      c #8A4472",
"*      c #F576C4",
"=      c #B71F71",
"-      c #B6829A",
";      c #D2BCB9",
">      c #D147A0",
",      c #3B0A18",
"'      c #4E4F25",
")      c #DD5EB8",
"!      c #E982AE",
"                  O)@@)!+**=OOO+                  ",
"              OO==+++++;++@++++!*;;               ",
"            ++@!++!*++++++@@!*!@@@)+;>>           ",
"         ,->@**!;@)*!+++@+@+@@+@++;+=;;>>>O       ",
"        ;;>>@++@@>@@@@++>+!!@+@+>>==;==>O>;)      ",
"      );>>>++++*!+++!++++@@@@)+>)=))=;O;=!->;     ",
"     ;,O&O>=*;++;)+++@+))*)!;+;+;+;+;=*->!>;)+    ",
"    >,,*&>>;==*>+++;+++)**>+;++;**@*;*='O@$O%>    ",
"   +O).O=O!>>+*=+*>**>*===>*+;++;++@>=)=>!>!%=)   ",
"   ,,&,>>=>++>++;++****+=+;=*;++@@+@>>-@=>!$=,=)  ",
" !&);=;O>O@.@@O+@;+$.,,O>O,O,,=@=,,=OO*'==>,>%,,>>",
")),>+.=O,)>!=>,>++!+O&,,,,,,,,@++'O=O>)+=>@,,=>,=)",
"O+;);,>>*=@>>+,.++)+$O,,,,,,@$@+;,%&@++%!$>,,.%=!)",
"&'=,,,.=!O>@!@,,=>%,,,.,,,,>*&)>,,=>+@+=$>-.=$>%>>",
"--;>%.+;!;>>@;,O,=!;,=,,,,,@)&>@,,,>+++>>!!&=>O@& ",
" >)=).>->*-@@+O,,+%>;,OO,,>@&>@%$,>)+++>!>>>)O%>  ",
"   >->%=>%>*+>,,O+!;>@%=-O)!+@>>+.>%=+;!!=O,->,   ",
"    O>>;O>$++!-,>!)!@+%=>@>*=>%)-%,>++;)>->!=     ",
"     )>;->>)@);,!>@>*@@;O-@'>---);=)++&>!)),      ",
"      >>%>+>)->&>*+)*!@*+)+;>-;+>)!>);)$,,,       ",
"        >O,%%>->;-@@);;;;!>!@!@!>@))O,,=,,        ",
"         >%>-,O=>>OO===>!>O,,=OO>>&,,,,,,         ",
"          >&,,.,=OO,,,,,=,,,,,.>%,,,,,O,          ",
"           >+,,,,=>O,,,,,,,,,,=>,.,>+>,           ",
"             )O,,,%,=%>-%O=>%,O>$,,,,             ",
"               >-=->=,=!=O;;,>!,%>=,              ",
"                =O;,,,,,>O>,,,,>,,,               ",
"                 $,=,==*$O,,,.;,,,                ",
"                  &,=>==,=O,,+.,>                 ",
"                   %,OO,)>!>-$,;                  ",
"                    *);;;!+-@);                   ",
"                      ););O-)                     ",
"                       >++O+                      ",
"                        >O+                       ",
"                         >                        "
    ]
    
      def get_logo(colormap)
        pix, mask = Gdk::Pixmap::create_from_xpm_d(self.window, nil, LOGO_IMAGE)
        return Gtk::Pixmap::new(pix, mask)
      end

    end
  end

###############################################################################

end
