#!/usr/bin/env ruby
=begin header
CLOTHIFY version 1.02
Gives the current layer in the indicated image a cloth-like texture.
Process invented by Zach Beane (Xath@irc.gimp.net)

Tim Newsome <drz@froody.bloke.com> 4/11/97

Translated into Ruby by Masahiro Sakai <zvm01052@nifty.ne.jp>, 26th Feb 2001
=end

require "gimp"
require "gimpfu"
include Gimp

def ruby_clothify(timg, tdrawable, bx=9, by=9,
		  azimuth=135, elevation=45, depth=3)
  width  = tdrawable.width
  height = tdrawable.height

  img = Image::new(width, height, RGB)
  begin
    img.undo_disable()

    layer_one = Layer::new(img, width, height, RGB_IMAGE, "X Dots")

    if check_version(1, 1, 0)
      edit_fill(layer_one, BG_IMAGE_FILL)
    else
      edit_fill(img, layer_one)
    end
    img.add_layer(layer_one, 0)
    PDB.plug_in_noisify(RUN_NONINTERACTIVE,
			img, layer_one, false, 0.7, 0.7, 0.7, 0.7)
    layer_two = layer_one.copy(true).set_mode(MULTIPLY_MODE).set_name("Y Dots")
    img.add_layer(layer_two, 0)
    PDB.plug_in_gauss_rle(RUN_NONINTERACTIVE, img, layer_one, bx, 1, 0)
    PDB.plug_in_gauss_rle(RUN_NONINTERACTIVE, img, layer_two, by, 0, 1)
    img.flatten()
    bump_layer = img.get_active_layer
    PDB.plug_in_c_astretch(RUN_NONINTERACTIVE, img, bump_layer)
    PDB.plug_in_noisify(RUN_NONINTERACTIVE,
			img, bump_layer, false, 0.2, 0.2, 0.2, 0.2)
    PDB.plug_in_bump_map(RUN_NONINTERACTIVE,
			 img, tdrawable, bump_layer, azimuth,
			 elevation, depth, 0, 0, 0, 0, true, false, 0)
  ensure
    img.delete
  end
end


register("ruby_fu_clothify",
	 "Make the specified layer look like it is printed on cloth",
	 "Make the specified layer look like it is printed on cloth",
	 "James Henstridge",
	 "James Henstridge",
	 "1997-1999",
	 "<Image>/Ruby-Fu/Alchemy/Clothify",
	 "RGB*, GRAY*",
	 [
	   [RF_IMAGE,    "image", "Image"],
	   [RF_DRAWABLE, "drawable", "Drawable"],
	   [RF_ADJUSTMENT, "x_blur", "X Blur", 9, [3, 100, 1, 10, 0, 1]],
	   [RF_ADJUSTMENT, "y_blur", "Y Blur", 9, [3, 100, 1, 10, 0, 1]],
	   [RF_ADJUSTMENT, "azimuth", "Azimuth", 135, [0, 360, 1, 10, 1, 0]],
	   [RF_ADJUSTMENT, "elevation", "elevation", 45, [0, 90, 1, 10, 1, 0]],
	   [RF_ADJUSTMENT, "depth", "Depth", 3, [1, 50, 1, 10, 0, 1]]
	]
) { |*args|
  ruby_clothify(*args)
}

main()
