#!/usr/bin/env ruby

=begin header
The GIMP -- an image manipulation program
Copyright (C) 1995 Spencer Kimball and Peter Mattis

GIMP raindrop effect Script  for GIMP 1.2
Copyright (C) 2001 Iccii <iccii@hotmail.com>
Copyright (C) 2001 Masahiro Sakai <zvm01052@nifty.ne.jp>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.  

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Changelog

raindrop.rb v0.1 by Masahiro Sakai 2001/02/26
  Translated into Ruby.

Raindrop v0.1 by Iccii 2001/01/30
  First relase
  This is "FIXME" version, which is spaghetti program code
=end


require "gimp"
require "gimpfu"
include Math
include Gimp


def apply_raindrop_logo_effect(img, logo_layer,
			       text_color, bg_color, light, blur,
			       hi_width, hi_offset, sh_offset, antialias)

  old_fg = palette_get_foreground
  old_bg = palette_get_background

  begin
    width  = logo_layer.width
    height = logo_layer.height

    radians = light * PI / 180
    hi_xoffset = - hi_offset * cos(radians)
    hi_yoffset = - hi_offset * sin(radians)
    sh_xoffset = sh_offset * cos(radians)
    sh_yoffset = sh_offset * sin(radians)

    # $BGX7J%l%$%d!<$K$OGX7J?'F~$l!"%Y!<%9%l%$%d!<$K$OJ8;zF~$l(B
    bg_layer = Layer::new(img, width, height, RGBA_IMAGE, "BG layer")
    img.resize(width, height, 0, 0)
    img.add_layer(bg_layer, 1)
    palette_set_background(bg_color)
    edit_fill(bg_layer, BG_IMAGE_FILL)
    palette_set_foreground(Color::new(255, 255, 255))
    palette_set_background(Color::new(  0,   0,   0)) 
    logo_layer.set_preserve_trans(true)
    edit_fill(logo_layer, FG_IMAGE_FILL)
    selection_layer_alpha(logo_layer)
    if (antialias)
      selection_feather(img, 3)
    end
    selection_invert(img)
    logo_layer.set_preserve_trans(false)
    edit_fill(logo_layer, BG_IMAGE_FILL)
    selection_none(img)
    
    # $B%l%$%d!<$r%3%T!<$7$F=`Hw(B
    layers = {}
    layers[:value]       = logo_layer.copy(true).set_name("Value layer").set_mode(ADDITION_MODE).set_opacity(15)
    layers[:hilight]     = logo_layer.copy(true).set_name("Highlight layer").set_mode(SCREEN_MODE).set_opacity(90)
    layers[:hilight_inn] = logo_layer.copy(true).set_name("Highlight inner").set_mode(SCREEN_MODE).set_opacity(90)
    layers[:shadow]      = logo_layer.copy(true).set_name("Drop shadow").set_mode(SUBTRACT_MODE).set_opacity(80)
    layers[:shadow_inn]  = logo_layer.copy(true).set_name("Inner shadow").set_mode(DARKEN_ONLY_MODE).set_opacity(80)
    layers[:color]       = logo_layer.copy(true).set_name("Color adjust")
    img.add_layer(layers[:color], 1)
    img.add_layer(layers[:shadow_inn], 1)
    img.add_layer(layers[:shadow], 1)
    img.add_layer(layers[:hilight_inn], 1)
    img.add_layer(layers[:hilight], 1)
    img.add_layer(layers[:value], 1)

    # $B%l%$%d!<%^%9%/$rDI2C$9$k(B
    selection_all(img)
    edit_copy(logo_layer)
    selection_none(img)
    masks = {}
    [:hilight, :hilight_inn, :shadow, :shadow_inn, :color].each{ |item|
      masks[item] = layers[item].create_mask(WHITE_MASK)
      img.add_layer_mask(layers[item], masks[item])
      floating_sel_anchor(edit_paste(masks[item], false))
    }

    # $BH?E>!"?'IU$1(B
    invert(layers[:hilight_inn])
    invert(masks[:shadow])
    selection_all(img)
    palette_set_foreground(text_color)
    edit_fill(layers[:color], FG_IMAGE_FILL)
    selection_none(img)

    # $B%l%$%d!<$r$:$i$7$F$\$+$9(B
    channel_ops_offset(layers[:hilight_inn], 0, 0, - sh_xoffset, - sh_yoffset)
    channel_ops_offset(masks[:shadow], 0, 0, sh_xoffset, sh_yoffset)
    channel_ops_offset(masks[:shadow_inn], 0, 0, sh_xoffset, sh_yoffset)
    PDB.plug_in_gauss_iir2(RUN_NONINTERACTIVE,
			   img, layers[:hilight_inn], blur, blur)
    PDB.plug_in_gauss_iir2(RUN_NONINTERACTIVE,
			   img, layers[:shadow], blur, blur)
    PDB.plug_in_gauss_iir2(RUN_NONINTERACTIVE,
			   img, layers[:shadow_inn], blur, blur)

    # $B%O%$%i%$%H%l%$%d!<$N=hM}(B
    channel_ops_offset(layers[:hilight], 0, 0, hi_xoffset, hi_yoffset)
    channel_ops_offset(masks[:hilight], 0, 0, hi_xoffset, hi_yoffset)
    selection_load(masks[:hilight])
    selection_sharpen(img)
    hilight_width = 0
    while (selection_is_empty(img) == 0) 
      selection_shrink(img, 1)
      hilight_width += 1
    end
    selection_none(img)
    selection_load(masks[:hilight])
    selection_invert(img)
    selection_grow(img, (100 - hi_width) * hilight_width / 100)
    selection_feather(img, 0.8 * blur)
    palette_set_foreground(Color::new(0, 0, 0))
    edit_fill(layers[:hilight], FG_IMAGE_FILL)
    selection_none(img)

    # $B=*$j(B
    img.remove_layer(logo_layer)
    img.set_active_layer(layers[:value])
  ensure
    palette_set_background(old_bg)
    palette_set_foreground(old_fg)
  end
end


def raindrop_logo_alpha(img, text_layer, text_color, bg_color,
			light, hi_width, hi_offset, sh_offset, blur, antialias)
  img.undo_group {
    img.remove_layer_mask(text_layer, APPLY) if text_layer.mask
    apply_raindrop_logo_effect(img, text_layer, text_color, bg_color, light,
			       hi_width, hi_offset, sh_offset, blur, antialias)
  }
  displays_flush()
end


# $B%m%4:n@.(B
def raindrop_logo(text, font_size, fontname, text_color, bg_color,
		  light, hi_width, hi_offset, sh_offset, blur, antialias)
  img = Image::new(256, 256, RGB)
  text_layer = text_fontname(img, -1, 0, 0, text, 20, true, font_size, PIXELS, fontname)

  img.undo_disable
  begin
    text_layer.set_name(text)
    apply_raindrop_logo_effect(img, text_layer, text_color, bg_color, light,
			       hi_width, hi_offset, sh_offset, blur, antialias)
  ensure
    img.undo_enable
  end
  Display::new(img)
end


register("ruby_fu_raindrop_logo_alpha2",
	 "Creates a logo like a raindrop",
	 "Creates a logo like a raindrop", 
	 "Iccii <iccii@hotmail.com>",
	 "Copyright 2001, Iccii",
	 "Feb, 2001",
	 "<Image>/Ruby-Fu/Alpha to Logo/Rain drop...",
	 "RGBA",
	 [
	   [RF_IMAGE,      "image", "Image"],
	   [RF_DRAWABLE,   "drawable", "Drawable"],
	   [RF_COLOR,      "text_color", "Text Color",     Color::new(0, 63, 255)],
	   [RF_COLOR,      "bg_color", "Background Color", Color::new(0, 127, 255)],
	   [RF_ADJUSTMENT, "light", "Lighting (degrees)",  45, [0, 360, 1, 10, 0, 0]],
	   [RF_ADJUSTMENT, "blur", "Blur Radius",          10, [1, 63, 1, 5, 0, 0]],
	   [RF_ADJUSTMENT, "hilight_width", "Highlight Width (%)",  30, [0, 100, 1, 5, 0, 0]],
	   [RF_ADJUSTMENT, "hilight_offset", "Highlight Offset", 3, [0, 50, 1, 5, 0, 1]],
	   [RF_ADJUSTMENT, "shadow_offset", "Shadow Offset", 5, [0, 50, 1, 5, 0, 1]],
	   [RF_TOGGLE,     "antialias", "Antialias",            true]
	 ]
) { |*args|
  raindrop_logo_alpha(*args)
}

register("ruby_fu_raindrop_logo",
	 "Creates a logo like a raindrop",
	 "Creates a logo like a raindrop", 
	 "Iccii <iccii@hotmail.com>",
	 "Copyright 2001, Iccii",
	 "Feb, 2001",
	 "<Toolbox>/Xtns/Ruby-Fu/Logos/Rain drop...",
	 "",
	 [
	   [RF_STRING,      "text", "Text", "Rain Drop"],
	   [RF_ADJUSTMENT,  "font_size", "Font Size (pixeld)", 150,
	     [2, 1000, 1, 10, 0, 1]],
	   [RF_FONT, "font", "Font", "-*-slogan-*-r-*-*-24-*-*-*-p-*-*-*"],
	   [RF_COLOR,      "text_color", "Text Color",
	     Color::new(0, 63, 255)],
	   [RF_COLOR,      "bg_color", "Background Color",
	     Color::new(0, 127, 255)],
	   [RF_ADJUSTMENT, "light", "Lighting (degrees)",  45,
	     [0, 360, 1, 10, 0, 0]],
	   [RF_ADJUSTMENT, "blur", "Blur Radius",          10,
	     [1, 63, 1, 5, 0, 0]],
	   [RF_ADJUSTMENT, "hilight_width", "Highlight Width (%)",  30,
	     [0, 100, 1, 5, 0, 0]],
	   [RF_ADJUSTMENT, "hilight_offset", "Highlight Offset",     3,
	     [0, 50, 1, 5, 0, 1]],
	   [RF_ADJUSTMENT, "shadow_offset", "Shadow Offset",        5,
	     [0, 50, 1, 5, 0, 1]],
	   [RF_TOGGLE,     "antialias", "Antialias",            true]
	 ]
) { |*args|
  raindrop_logo(*args)
}

main()
