#!/usr/bin/env ruby
require "gimp"
require "gimpfu"
include Gimp

=begin
Translated into Ruby by Masahiro Sakai <zvm01052@nifty.ne.jp>, 26th Feb 2001
=end

=begin header
Chris Gutteridge (cjg@ecs.soton.ac.uk)
At ECS Dept, University of Southampton, England.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=end


def render_map(inSize, inGrain, inGrad, inWiden)
  old_gradient = gradients_get_active
  old_fg = palette_get_foreground
  old_bg = palette_get_background
  begin
    theWidth  = inSize
    theHeight = inSize
    theImage  = Image::new(theWidth, theHeight, RGB)

    selection_all(theImage)

    theLayer = Layer::new(theImage, theWidth, theHeight, RGBA_IMAGE,
			  "I've got more rubber ducks than you!")
    theImage.add_layer(theLayer, 0)
    PDB.plug_in_solid_noise(RUN_NONINTERACTIVE, theImage, theLayer, 1, 0,
			    rand(65536), inGrain, inGrain, inGrain)

    if inWiden == 1
      thinLayer = Layer::new(theImage, theWidth, theHeight, RGBA_IMAGE,
			     "Camo Thin Layer")
      theImage.add_layer(thinLayer, 0)
      theBigGrain = min(15, s * inGrain)
      PDB.plug_in_solid_noise(RUN_NONINTERACTIVE, theImage, thinLayer, 1, 0,
			      rand(65536),
			      theBigGrain, theBigGrain, theBigGrain)
      palette_set_background(Color::new(255, 255, 255))
      palette_set_foreground(Color::new(0, 0, 0))
      theMask = thinLayer.create_mask(WHITE_MASK)
      theImage.add_layer_mask(thinLayer, theMask)
      blend(theMask, FG_BG_RGB, NORMAL_NODE, LINEAR, 100, 
	    0, REPEAT_TRIANGULAR, false, 0, 0, 0, 0, theHeight / 2)
      theLayer = theImage.flatten()
    end

    selection_none(theImage)
    gradients_set_active(inGrad)
    PDB.plug_in_gradmap(RUN_NONINTERACTIVE, theImage, theLayer)
  ensure
    gradients_set_active(old_gradient)
    palette_set_background(old_bg)
    palette_set_foreground(old_fg)
  end
  Display::new(theImage)
end


# Register the function with the GIMP:

register("ruby_fu_render_map",
	 "Another pattern which resembles a map",
	 "Another pattern which resembles a map",
	 "Chris Gutteridge: cjg@ecs.soton.ac.uk",
	 "Chris Gutteridge / ECS @ University of Southampton, England",
	 "28th April 1998",
	 "<Toolbox>/Xtns/Ruby-Fu/Patterns/Render Map...",
	 "",
	 [
	   [RF_ADJUSTMENT, "image_size", "Image Size", 256,
	     [0, 2048, 1, 10, 0, 0]],
	   [RF_ADJUSTMENT, "granularity", "Granularity", 4,
	     [0, 15, 1, 1, 0, 0]],
	   [RF_GRADIENT, "gradient", "Gradient", "Land_and_Sea"],
	   [RF_OPTION, "behaviour", "Behaviour", 0,
	     ["Tile", "Detail in Middle"]],
	 ],
	 []
) {
  |*args| 
  render_map(*args)
}

main()
